@@ -44,6 +44,18 @@ context can be accessed through this instance from the attributes ``data`` and `
     in your commands. Instead of converting the arguments, the raw, unconverted arguments will be passed back
     to the command.
 """
+from __future__ import annotations
+
+__all__: typing.Final[typing.List[str]] = [
+    "WrappedArg",
+    "user_converter",
+    "member_converter",
+    "text_channel_converter",
+    "voice_channel_converter",
+    "category_converter",
+    "role_converter",
+]
+
 import re
 import typing
 import collections
@@ -53,16 +65,6 @@ import hikari
 from lightbulb import context
 from lightbulb import errors
 
-__all__: typing.Final[typing.Tuple[str]] = (
-    "WrappedArg",
-    "user_converter",
-    "member_converter",
-    "text_channel_converter",
-    "voice_channel_converter",
-    "category_converter",
-    "role_converter",
-)
-
 USER_MENTION_REGEX: typing.Final[typing.Pattern] = re.compile(r"<@!?(\d+)>")
 CHANNEL_MENTION_REGEX: typing.Final[typing.Pattern] = re.compile(r"<#(\d+)>")
 ROLE_MENTION_REGEX: typing.Final[typing.Pattern] = re.compile(r"<@&(\d+)>")
@@ -94,7 +96,16 @@ def _resolve_id_from_arg(arg_string: str, regex: typing.Pattern) -> hikari.Snowf
     return hikari.Snowflake(arg_string)
 
 
-async def user_converter(arg: WrappedArg) -> hikari.UserImpl:
+async def _get_or_fetch_guild_channel_from_id(arg: WrappedArg, channel_id: hikari.Snowflake) -> hikari.GuildChannel:
+    channel = None
+    if arg.context.bot._has_stateful_cache:
+        channel = arg.context.bot.cache.get_guild_channel(channel_id)
+    if channel is None:
+        channel = await arg.context.bot.rest.fetch_channel(channel_id)
+    return channel
+
+
+async def user_converter(arg: WrappedArg) -> hikari.User:
     """
     Converter to transform a command argument into a :obj:`hikari.models.users.UserImpl` object.
 
@@ -115,8 +126,10 @@ async def user_converter(arg: WrappedArg) -> hikari.UserImpl:
             async def username(ctx, user: lightbulb.converters.user_converter):
                 await ctx.reply(user.username)
     """
-    # TODO - Implement cache checking
     user_id = _resolve_id_from_arg(arg.data, USER_MENTION_REGEX)
+    if arg.context.bot._has_stateful_cache:
+        if (user := arg.context.bot.cache.get_user(user_id)) is not None:
+            return user
     return await arg.context.bot.rest.fetch_user(user_id)
 
 
@@ -135,6 +148,9 @@ async def member_converter(arg: WrappedArg) -> hikari.Member:
     """
     # TODO - Implement cache checking
     user_id = _resolve_id_from_arg(arg.data, USER_MENTION_REGEX)
+    if arg.context.bot._has_stateful_cache:
+        if (member := arg.context.bot.cache.get_member(arg.context.guild_id, user_id)) is not None:
+            return member
     return await arg.context.bot.rest.fetch_member(arg.context.guild_id, user_id)
 
 
@@ -155,7 +171,7 @@ async def text_channel_converter(arg: WrappedArg) -> typing.Union[hikari.GuildTe
     """
     # TODO - Implement cache checking
     channel_id = _resolve_id_from_arg(arg.data, CHANNEL_MENTION_REGEX)
-    channel = await arg.context.bot.rest.fetch_channel(channel_id)
+    channel = await _get_or_fetch_guild_channel_from_id(arg, channel_id)
     if not isinstance(channel, (hikari.GuildTextChannel, hikari.GuildNewsChannel)):
         raise errors.ConverterFailure("Channel is not a text channel")
     return channel
@@ -176,7 +192,7 @@ async def voice_channel_converter(arg: WrappedArg) -> hikari.GuildVoiceChannel:
     """
     # TODO - Implement cache checking
     channel_id = _resolve_id_from_arg(arg.data, CHANNEL_MENTION_REGEX)
-    channel = await arg.context.bot.rest.fetch_channel(channel_id)
+    channel = await _get_or_fetch_guild_channel_from_id(arg, channel_id)
     if not isinstance(channel, hikari.GuildVoiceChannel):
         raise errors.ConverterFailure("Channel is not a voice channel")
     return channel
@@ -197,7 +213,7 @@ async def category_converter(arg: WrappedArg) -> hikari.GuildCategory:
     """
     # TODO - Implement cache checking
     channel_id = _resolve_id_from_arg(arg.data, CHANNEL_MENTION_REGEX)
-    channel = await arg.context.bot.rest.fetch_channel(channel_id)
+    channel = await _get_or_fetch_guild_channel_from_id(arg, channel_id)
     if not isinstance(channel, hikari.GuildCategory):
         raise errors.ConverterFailure("Channel is not a voice channel")
     return channel
@@ -218,6 +234,9 @@ async def role_converter(arg: WrappedArg) -> hikari.Role:
     """
     # TODO - Implement cache checking
     role_id = _resolve_id_from_arg(arg.data, CHANNEL_MENTION_REGEX)
+    if arg.context.bot._has_stateful_cache:
+        if (role := arg.context.bot.cache.get_role(role_id)) is not None:
+            return role
     roles = await arg.context.bot.rest.fetch_roles(arg.context.guild_id)
     roles = dict([(r.id, r) for r in roles])
     return roles[role_id]
