@@ -16,17 +16,8 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with Lightbulb. If not, see <https://www.gnu.org/licenses/>.
 from __future__ import annotations
-import abc
-import typing
 
-from hikari.events import base
-
-if not typing.TYPE_CHECKING:
-    import hikari
-
-    from lightbulb import commands
-
-__all__: typing.Final[typing.Tuple[str]] = (
+__all__: typing.Final[typing.List[str]] = [
     "CommandErrorEvent",
     "LightbulbError",
     "ExtensionError",
@@ -47,17 +38,27 @@ __all__: typing.Final[typing.Tuple[str]] = (
     "OnlyInGuild",
     "OnlyInDM",
     "NotOwner",
-)
+]
+
+import abc
+import attr
+import typing
+
+from hikari.events import base_events
+
+if typing.TYPE_CHECKING:
+    import types
+
+    import hikari
+
+    from lightbulb import commands
 
 
-class CommandErrorEvent(base.Event):
+@attr.s(kw_only=True, slots=True)
+class CommandErrorEvent(base_events.Event):
     """
     Event type to subscribe to for the processing of all command errors raised by the handler.
 
-    Args:
-        error (:obj:`~.errors.CommandError`): An instance or subclass of ``CommandError``. The error that was raised.
-        message (:obj:`hikari.models.messages.Message`): The message that caused the exception to be raised.
-
     Example:
 
         .. code-block:: python
@@ -72,10 +73,19 @@ class CommandErrorEvent(base.Event):
 
     """
 
-    def __init__(self, error: Exception, message: hikari.Message) -> None:
-        self.error = error
-        self.traceback = error.__traceback__
-        self.message = message
+    app: hikari.api.event_consumer.IEventConsumerApp = attr.ib()
+    """App instance for this application."""
+    exception: LightbulbError = attr.ib()
+    """The exception that triggered this event."""
+    message: hikari.Message = attr.ib()
+    """The message that this event was triggered for."""
+    command: typing.Optional[commands.Command] = attr.ib(default=None)
+    """The command that this event was triggered for."""
+
+    @property
+    def traceback(self) -> types.TracebackType:
+        """The traceback for this event's exception."""
+        return self.exception.__traceback__
 
 
 class LightbulbError(Exception):
@@ -243,6 +253,22 @@ class NotOwner(CheckFailure):
     pass
 
 
+class BotOnly(CheckFailure):
+    """
+    Error raised when the command invoker is not a bot.
+    """
+
+    pass
+
+
+class HumanOnly(CheckFailure):
+    """
+    Error raised when the command invoker is not an human.
+    """
+
+    pass
+
+
 class MissingRequiredRole(CheckFailure):
     """
     Error raised when the member invoking a command is missing one or more role required.
